package game;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;
import main.Art;
import main.Art.Bitmap;
import main.GameComponent;
import main.InputHandler;
import entity.Building;
import entity.Entity.ID;
import entity.Unit;
import game.MainMenu.MenuOption;

public class DeadGame {
	
	public enum Difficulty {
		AI_EASY(300, Art.ai_easy),
		AI_NORMAL(200, Art.ai_normal),
		AI_HARD(150, Art.ai_hard),
		AI_HARDER(100, Art.ai_harder);
		
		public int value;
		public Bitmap bitmap;
		
		private Difficulty(int value, Bitmap bitmap) {
			this.value = value;
			this.bitmap = bitmap;
		}
	}
	
	public InputHandler inputHandler;
	public ArrayList<Building> buildings;
	public ArrayList<Unit> units;
	public MainMenu mainMenu;
	//public int[] data;
	public ArrayList<PixelData> pixelDataList;
	
	private boolean isInitialized;
	private boolean menuGameStartFlag; //Used to determine if the player clicks on "Start" in the main menu.
	private boolean gameOverFlag;
	private boolean greenWins;
	private boolean fadeToBlack;
	private boolean doneFadingToBlack;
	private boolean retryFlag; //Asks player if they want to retry.
	
	private int timerCountdown = 0;
	private int playerCountdown = 0;
	private Difficulty difficulty;
	private GameComponent gameComponent;
	
	public DeadGame(GameComponent g) {
		gameComponent = g;
		inputHandler = new InputHandler();
		buildings = new ArrayList<Building>();
		units = new ArrayList<Unit>();
		difficulty = Difficulty.AI_EASY;
		mainMenu = new MainMenu(this);
		//data = g.pixels;
		pixelDataList = new ArrayList<PixelData>();
		for (int i = 0; i < g.pixels.length; i++) {
			int x = i % GameComponent.MAGNIFIED;
			int y = i / GameComponent.MAGNIFIED;
			PixelData pixelData = new PixelData();
			pixelData.color = g.pixels[i];
			pixelData.x = x;
			pixelData.y = y;
			pixelDataList.add(pixelData);
		}
		
		g.addMouseListener(inputHandler);
		g.addMouseMotionListener(inputHandler);
		
		//		menuGameStartFlag = false;
		//		gameOverFlag = false;
		//		greenWins = false; //Default: Red always win.
		//		fadeToBlack = false; //Doesn't change value until losing condition is met.
		//		doneFadingToBlack = false; //Doesn't change value until finished fading to black.
		
		isInitialized = false;
		initialize();
	}
	
	public void tick() {
		if (menuGameStartFlag) {
			if (!isInitialized)
				initialize();
			else if (!gameOverFlag) {
				if (playerCountdown > 0)
					playerCountdown--;
				if (inputHandler.mouseClicked) {
					inputHandler.mouseClicked = false;
					if (playerCountdown <= 0) {
						boolean hasBuilding = false;
						for (Building b : buildings) {
							if (inputHandler.mouseX == b.x && inputHandler.mouseY == b.y) {
								hasBuilding = true;
								break;
							}
						}
						if (hasBuilding == false) {
							Building b = new Building();
							b.x = inputHandler.mouseX;
							b.y = inputHandler.mouseY;
							b.setColor(0xFF00FF00);
							b.setID(ID.BUILDING);
							b.setNumber(buildings.size() + 1);
							buildings.add(b);
							playerCountdown = 100;
						}
					}
				}
				for (Iterator<Building> i = buildings.iterator(); i.hasNext();) {
					Building b = i.next();
					b.tick();
					if (b.shouldDespawn())
						i.remove();
					if (b.isSendingUnitOut()) {
						units.add(b.createUnit());
						b.finishedSendingUnitOut();
					}
				}
				for (Iterator<Unit> i = units.iterator(); i.hasNext();) {
					Unit u = i.next();
					u.findNearestEntity(buildings, units);
					u.tick();
					if (u.shouldDespawn()) {
						i.remove();
					}
				}
				randomEnemyBuilding();
				losingCondition();
			}
			else if (doneFadingToBlack && gameOverFlag) {
				//uninitialize();
				//initialize();
				uninitialize();
				retryFlag = true;
				returnToMenu();
			}
		}
		else {
			mainMenu.tick();
		}
	}
	
	public void setRetryFlag() {
		retryFlag = true;
	}
	
	public boolean isAskingForRetry() {
		return retryFlag;
	}
	
	public void confirmRetry() {
		//Player pressed "Retry?"
		retryFlag = false;
		setGameMenuFlag(MenuOption.START);
		initialize();
	}
	
	public void cancelRetry() {
		//Player pressed "No".
		retryFlag = false;
		returnToMenu();
	}
	
	public void render() {
		if (menuGameStartFlag) {
			if (!gameOverFlag) {
				clear();
				renderBuildings();
				renderUnits();
				renderMouseCursor();
			}
			else {
				fade();
			}
		}
		else {
			clear();
			mainMenu.render();
		}
		renderBack();
	}
	
	private void renderBack() {
		for (int i = 0; i < pixelDataList.size(); i++)
			gameComponent.pixels[i] = pixelDataList.get(i).color;
	}
	
	public int getPlayerTimerCountdown() {
		//To notify when the player is able to put down spawners.
		return playerCountdown;
	}
	
	public int getEnemyTimerCountdown() {
		//To notify when the computer is putting down spawners.
		return timerCountdown;
	}
	
	public boolean hasGameStarted() {
		return menuGameStartFlag;
	}
	
	public boolean isGameOver() {
		return gameOverFlag;
	}
	
	public void setComputerDifficulty(Difficulty diff) {
		this.difficulty = diff;
	}
	
	public Difficulty getComputerDifficulty() {
		return difficulty;
	}
	
	public void setGameMenuFlag(MenuOption options) {
		//Temporarily commented out, just in case I'm expanding this.
		//		switch (options) {
		//			case START:
		//				menuGameStartFlag = true;
		//				break;
		//			case BACK:
		//				break;
		//			case DIFFICULTY:
		//				break;
		//			case NO:
		//				break;
		//			case OPTIONS:
		//				break;
		//			case YES:
		//				break;
		//			default:
		//				break;
		//		}
		menuGameStartFlag = true;
	}
	
	private void renderMouseCursor() {
		int x = inputHandler.mouseX / GameComponent.SCALE;
		int y = inputHandler.mouseY / GameComponent.SCALE;
		int length = y * (GameComponent.MAGNIFIED) + x;
		if (length < pixelDataList.size() && length >= 0)
			pixelDataList.get(length).color = -1;
	}
	
	private void renderBuildings() {
		for (Building b : buildings) {
			b.render(pixelDataList);
		}
	}
	
	private void renderUnits() {
		for (Unit u : units)
			u.render(pixelDataList);
	}
	
	public void initialize() {
		
		buildings.clear();
		units.clear();
		gameOverFlag = false;
		greenWins = false; //Default: Red always win.
		fadeToBlack = false; //Doesn't change value until losing condition is met.
		doneFadingToBlack = false; //Doesn't change value until finished fading to black.
		
		Random random = new Random();
		int green = random.nextInt(3) + 1;
		int red = random.nextInt(3) + 1;
		
		for (int g = 0; g < green; g++) {
			Building b = new Building();
			b.x = random.nextInt(GameComponent.WIDTH);
			b.y = random.nextInt(GameComponent.HEIGHT);
			boolean hasBuilding = false;
			for (Building c : buildings) {
				if (c.x == b.x && c.y == b.y) {
					hasBuilding = true;
					break;
				}
			}
			if (hasBuilding == false) {
				b.setColor(0xFF00FF00);
				buildings.add(b);
			}
			else
				g--;
		}
		for (int g = 0; g < red; g++) {
			Building b = new Building();
			b.x = random.nextInt(GameComponent.WIDTH);
			b.y = random.nextInt(GameComponent.HEIGHT);
			boolean hasBuilding = false;
			for (Building c : buildings) {
				if (c.x == b.x && c.y == b.y) {
					hasBuilding = true;
					break;
				}
			}
			if (hasBuilding == false) {
				b.setColor(0xFFFF0000);
				buildings.add(b);
			}
			else
				g--;
		}
		playerCountdown = 0;
		timerCountdown = 0;
		inputHandler.mouseClicked = false;
		isInitialized = true;
	}
	
	public void uninitialize() {
		isInitialized = false;
	}
	
	public boolean isGameInitialized() {
		return isInitialized;
	}
	
	public void returnToMenu() {
		menuGameStartFlag = false;
	}
	
	private void losingCondition() {
		int red = 0;
		int green = 0;
		for (Building b : buildings) {
			if (b.getTeamColor() == 0xFFFF0000)
				red++;
			if (b.getTeamColor() == 0xFF00FF00)
				green++;
		}
		if (red == 0 || green == 0) {
			//Game over.
			gameOverFlag = true;
		}
		if (red == 0)
			greenWins = true;
	}
	
	private void randomEnemyBuilding() {
		timerCountdown--;
		if (timerCountdown <= 0) {
			timerCountdown = difficulty.value;
			Random r = new Random();
			boolean hasBuilding;
			int x, y;
			do {
				hasBuilding = false;
				x = r.nextInt(GameComponent.WIDTH);
				y = r.nextInt(GameComponent.HEIGHT);
				for (Building b : buildings) {
					if (x == b.x && y == b.y) {
						hasBuilding = true;
						break;
					}
				}
			}
			while (hasBuilding);
			Building b = new Building();
			b.x = x;
			b.y = y;
			b.setColor(0xFFFF0000);
			buildings.add(b);
		}
	}
	
	private void fade() {
		if (!fadeToBlack) {
			if (greenWins) {
				if (checkFadingColor(0xFF00FF00))
					fadeToBlack = true;
			}
			else {
				if (checkFadingColor(0xFFFF0000))
					fadeToBlack = true;
			}
		}
		else {
			if (checkFadingColor(0xFF000000))
				doneFadingToBlack = true;
		}
	}
	
	public void clear() {
		for (int i = 0; i < pixelDataList.size(); i++)
			pixelDataList.get(i).color = 0;
	}
	
	private boolean checkFadingColor(int valueToCheck) {
		boolean allSet = true;
		int temp = 0;
		int check = 0;
		for (int i = 0; i < pixelDataList.size(); i++) {
			if (pixelDataList.get(i).color != valueToCheck) {
				for (int bit = 24; bit >= 0; bit -= 8) {
					if (bit == 24) {
						temp = (pixelDataList.get(i).color & (0xFF << bit)) >> bit;
						if (temp == -1) {
							temp = 0xFF;
							pixelDataList.get(i).color = (pixelDataList.get(i).color & ~(0xFF << bit)) | (temp << bit);
						}
						else if (temp == 0xFF) {
							continue;
						}
						else {
							check = (valueToCheck & (0xFF << bit)) >> bit;
							if (temp != check) {
								if (check == -1) {
									check = 0xFF;
								}
								if (temp < check)
									temp++;
								else if (temp > check)
									temp--;
								pixelDataList.get(i).color = (pixelDataList.get(i).color & ~(0xFF << bit)) | (temp << bit);
							}
						}
						
					}
					else {
						check = (valueToCheck & (0xFF << bit)) >> bit;
						temp = (pixelDataList.get(i).color & (0xFF << bit)) >> bit;
						if (temp != check) {
							if (temp < check)
								temp++;
							else if (temp > check)
								temp--;
							pixelDataList.get(i).color = (pixelDataList.get(i).color & ~(0xFF << bit)) | (temp << bit);
						}
					}
				}
				allSet = false;
			}
		}
		return allSet;
	}
}
